/**
 * Copyright [2023] Jiří Štefka <jiriks74>
 * Project: AdventOfCode
 * @file main.c
 * @brief Main entry point
 * @author jiriks74
 */
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/**
 * @brief Gets the first number in a string
 * param String where you want to find the number
 * return The first number found (or -1 if none found)
 * Gets the first number in a string. Can find numbers written as digits and
 * words.
 */
int getNum(const char *line) {
  char *str = (char *)line;
  // Mapping of words to numeric representations
  const char *numbers[] = {"zero", "one", "two",   "three", "four",
                           "five", "six", "seven", "eight", "nine"};
  const char *digits[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};

  // Find first written number
  char *smallestPos = strchr(str, *"\0");
  int numAtSmallPos = -1;
  for (int i = 0; i < sizeof(numbers) / sizeof(numbers[0]); i++) {
    char *pos = NULL;

    char *written = strstr(str, numbers[i]);
    char *digit = strstr(str, digits[i]);

    if (written == NULL && digit == NULL)
      continue;
    if (written == NULL)
      pos = digit;
    else if (digit == NULL)
      pos = written;
    else
      pos = digit < written ? digit : written;

    if (pos < smallestPos) {
      numAtSmallPos = i;
      smallestPos = pos;
    }
  }

  // Chen which was first and return it
  return numAtSmallPos;
}

/**
 * @brief Gets a calibration value from a line
 * @param char* line containing the calibration value
 * @return int the calibration value
 */
int getVal(const char *str) {
  int result = 0;
  int result2 = -1;
  for (int i = 0; i < strlen(str); i++) {
    int num = getNum(&str[i]);
    if (num >= 0) {
      if (result == 0) {
        result = num * 10;
      } else {
        result2 = num;
      }
    }
  }
  if (result2 == -1) {
    result += result / 10;
  } else {
    result += result2;
  }
  return result;
}

/**
 * @brief Main entry point
 * @param argc Number of command-line arguments.
 * @param argv Array of command-line arguments.
 */
#ifdef TESTING
int mainTest(FILE *stdin, int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
  FILE *file;
  if (argc == 1) {
    file = stdin;
  } else {
    file = fopen(argv[1], "r"); // LCOV_EXCL_LINE
  }

  // LCOV_EXCL_START
  if (file == NULL) {
    if (argc > 1)
      fprintf(stderr, "Could not open file '%s'\n", argv[1]);
    else
      fprintf(stderr, "Couldn't open file\n");
    exit(EXIT_FAILURE);
  }
  // LCOV_EXCL_STOP

  char *line;
  size_t len = 0;
  ssize_t read;
  int result = 0;
  while ((read = getline(&line, &len, file) != -1)) {
    result += getVal(line);
  }
  printf("The sum of the calibration values is %d\n", result);
  return EXIT_SUCCESS;
}
